#include <config.h>

#include <stdio.h>
#ifndef __USE_BSD
# define __USE_BSD	/* Needed for mkdtemp () */
#endif

#include <stdlib.h>
#include <string.h>
#include <ctype.h>

#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <errno.h>
#include <fcntl.h>
#include <string.h>
#include <strings.h>

#ifdef ENABLE_NLS
# include <libintl.h>
#endif

#include "bmpx/util.h"
#include "bmpx/uri.h"
#include "bmpx/vfs.h"
#include "bmpx/plugin.h"
#include "bmpx/plugin_interfaces.h"

#include "libcontainer_pls.h"

#ifdef HAVE_FTS_H
#  include <fts.h>
#endif

static VFSHandle *handle = NULL;

gboolean
container_pls_can_process (BmpPluginContainerPLS *self, const gchar *uri)
{
    if (str_has_suffix_nocase (uri, ".pls")) 
	return TRUE;
    return FALSE;
}

gboolean
container_pls_handle_set (BmpPluginContainerPLS *self, VFSHandle *_handle)
{
    if (_handle == NULL) 
	return FALSE;
    
    if (!_handle->valid)
	return FALSE;

    handle = _handle;

    return TRUE;
}

VFSHandle*
container_pls_handle_get (void)
{
    return handle;
}

gboolean
container_pls_handle_remove (BmpPluginContainerPLS *self)
{
    handle = NULL;

    return TRUE;
}

gboolean 
container_pls_handle_read (BmpPluginContainerPLS *self)
{
    GList *uri_list = NULL;
    gchar *num_entries;
    gint num_entries_i, n;

    g_return_val_if_fail (handle != NULL, FALSE);
    g_return_val_if_fail (VFS_HANDLE_VALID(handle), FALSE);

    if ((!handle->buffer) || (!strlen(handle->buffer)))
      {
	handle->buffer = NULL;
	return TRUE;
      }

    num_entries =
	      read_ini_string_buffer (handle->buffer, "playlist", "numberofentries");
    num_entries_i =
	      strtol (num_entries, NULL, 10);
    g_free (num_entries);

    for (n = 0; n < num_entries_i; n++)
      {
	gchar *value, *key;

	key =
	    g_strdup_printf ("File%d", n+1);
	value =
	    read_ini_string_buffer (handle->buffer, "playlist", key);

	uri_list =
	    g_list_append (uri_list, g_strdup (value));
      }

    handle->uri_list = uri_list;
    return TRUE;
}

static void
container_pls_interface_init (gpointer         g_iface,
                              gpointer         iface_data)
{
        BmpPluginTransportInterfaceClass *klass = (BmpPluginTransportInterfaceClass *)g_iface;

	klass->can_process    = container_pls_can_process;
	klass->handle_set     = container_pls_handle_set;
	klass->handle_get     = container_pls_handle_get;
	klass->handle_remove  = container_pls_handle_remove;
	klass->handle_read    = container_pls_handle_read;
}

/* Metadata IFace */
static gchar* _metadata[] = {
    "PLS",
    "Milosz Derezynski",
    "(C) GPL 2005",
    "http://bmpx.berlios.de",
    "internalerror@gmail.com",
};

static const gchar**
get_metadata (BmpPlugMetadataInterface *self)
{
    return _metadata;
}

static void
metadata_init (gpointer         g_iface,
               gpointer         iface_data)
{
        BmpPlugMetadataInterfaceClass *klass = (BmpPlugMetadataInterfaceClass *)g_iface;

        klass->get_metadata = get_metadata;
}


GType plugin_container_pls_type = 0;

void 
plugin_container_pls_register_type (GTypeModule *module)
{
	    static const GTypeInfo object_info = {
                        sizeof (BmpPluginContainerPLSClass),
                        NULL,   /* base_init */
                        NULL,   /* base_finalize */
                        NULL,   /* class_init */
                        NULL,   /* class_finalize */
                        NULL,   /* class_data */
                        sizeof (BmpPluginContainerPLS),
                        0,      /* n_preallocs */
                        NULL    /* instance_init */
                };

                plugin_container_pls_type = g_type_module_register_type (module,
					     G_TYPE_OBJECT,
					     "BmpPluginContainerPLSType",
					     &object_info, 0);

                static const GInterfaceInfo interface_container_pls_info = {
                        (GInterfaceInitFunc) container_pls_interface_init,    /* interface_init */
                        NULL,                                       /* interface_finalize */
                        NULL                                        /* interface_data */
                };

                static const GInterfaceInfo interface_metadata_info = {
                        (GInterfaceInitFunc) metadata_init,    /* interface_init */
                        NULL,                                       /* interface_finalize */
                        NULL                                        /* interface_data */
                };

                g_type_module_add_interface (module,
					     plugin_container_pls_type,
                                             BMP_TYPE_PLUG_METADATA,
                                             &interface_metadata_info);


                g_type_module_add_interface (module,
					     plugin_container_pls_type,
                                             BMP_TYPE_PLUGIN_CONTAINER_IFACE,
                                             &interface_container_pls_info);
}
