#ifndef CONFIGURATION_H
#define CONFIGURATION_H

#include <glib-object.h>

G_BEGIN_DECLS

#define BMP_TYPE_CONFIGURATION		  (bmp_configuration_get_type ())
#define BMP_CONFIGURATION(obj)		  (G_TYPE_CHECK_INSTANCE_CAST ((obj), BMP_TYPE_CONFIGURATION, BmpConfiguration))
#define BMP_CONFIGURATION_CLASS(klass)	  (G_TYPE_CHECK_CLASS_CAST ((klass), BMP_TYPE_CONFIGURATION, BmpConfigurationClass))
#define BMP_IS_CONFIGURATION(obj)	  (G_TYPE_CHECK_INSTANCE_TYPE ((obj), BMP_TYPE_CONFIGURATION))
#define BMP_IS_CONFIGURATION_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), BMP_TYPE_CONFIGURATION))
#define BMP_CONFIGURATION_GET_CLASS(obj)  (G_TYPE_INSTANCE_GET_CLASS ((obj), BMP_TYPE_CONFIGURATION, BmpConfigurationClass))

typedef struct _BmpConfiguration BmpConfiguration;
typedef struct _BmpConfigurationClass BmpConfigurationClass;
typedef struct _BmpConfigurationPrivate BmpConfigurationPrivate;

struct _BmpConfiguration {
    GObject parent;

    BmpConfigurationPrivate *private;
};

struct _BmpConfigurationClass {
    GObjectClass parent;

    void (*removed) (BmpConfiguration * configuration);
    void (*changed) (BmpConfiguration * configuration);
    void (*added) (BmpConfiguration * configuration);
};

GType
bmp_configuration_get_type(void);

BmpConfiguration*
bmp_configuration_new(void);

gboolean
bmp_configuration_save(BmpConfiguration * configuration);

gboolean
bmp_configuration_domain_new(BmpConfiguration * configuration,
                             const gchar * domain_id);

gboolean
bmp_configuration_key_new(BmpConfiguration * configuration,
                          const gchar * domain_id,
                          const gchar * key_id,
                          GValue * key_value);

gboolean
bmp_configuration_key_set(BmpConfiguration * configuration,
                          const gchar * domain_id,
                          const gchar * key_id,
                          GValue * key_value,
                          gboolean create);

gboolean
bmp_configuration_key_unset (BmpConfiguration * configuration,
                         const gchar * domain_id,
                         const gchar * key_id,
                         gboolean create);

/* Simple API */

GHashTable*
bmp_configuration_domain_fetch(BmpConfiguration *configuration,
                               const gchar * domain_id);
void
bmp_configuration_ht_destroy(BmpConfiguration * configuration,
                             GHashTable * ht);


gchar*
bmp_configuration_key_fetch(BmpConfiguration * configuration,
                                       const gchar * domain_id,
                                       const gchar * key_id);
gint
bmp_configuration_key_fetch_int(BmpConfiguration * configuration,
                                       const gchar * domain_id,
                                       const gchar * key_id);
gboolean
bmp_configuration_key_fetch_boolean(BmpConfiguration * configuration,
                                    const gchar * domain_id,
                                    const gchar * key_id);

void
bmp_configuration_key_set_string (BmpConfiguration * configuration,
                                  const gchar * domain_id,
                                  const gchar * key_id,
                                  const gchar *value_arg);
void
bmp_configuration_key_set_int (BmpConfiguration * configuration,
                               const gchar * domain_id,
                               const gchar * key_id,
                               gint value_arg);
void
bmp_configuration_key_set_boolean (BmpConfiguration * configuration,
                                   const gchar * domain_id,
                                   const gchar * key_id,
                                   gboolean value_arg);


gboolean
bmp_configuration_domain_exists(BmpConfiguration * configuration,
                                const gchar * domain_id);
gboolean
bmp_configuration_domain_has_keys(BmpConfiguration * configuration,
                                  const gchar * domain_id);

gboolean
bmp_configuration_key_find(BmpConfiguration * configuration,
                           const gchar * domain_id,
                           const gchar * key_id, const gchar * key_value); /* Either key_id or key_value can be null, but of course not both */


gboolean
bmp_configuration_remove_domain(BmpConfiguration * configuration,
                                const gchar * domain_id);
gboolean
bmp_configuration_remove_key(BmpConfiguration * configuration,
                             const gchar * domain_id,
                             const gchar * key_id, const gchar * key_value);


#define LOOKUP_OR_CREATE_DEFAULT(domain_id,key_id,default_value) \
  if (!bmp_configuration_key_find (bmp_configuration, domain_id, key_id, NULL)) \
  { \
      bmp_configuration_key_set_string (bmp_configuration, domain_id, key_id, default_value); \
  }

#define LOOKUP_OR_CREATE_DEFAULT_INT(domain_id,key_id,default_value) \
  if (!bmp_configuration_key_find (bmp_configuration, domain_id, key_id, NULL)) \
  { \
      bmp_configuration_key_set_int (bmp_configuration, domain_id, key_id, default_value); \
  }

#define LOOKUP_OR_CREATE_DEFAULT_BOOLEAN(domain_id,key_id,default_value) \
  if (!bmp_configuration_key_find (bmp_configuration, domain_id, key_id, NULL)) \
  { \
      bmp_configuration_key_set_boolean (bmp_configuration, domain_id, key_id, default_value); \
  }

#endif /* CONFIGURATION_H */
